@@ -106,6 +106,15 @@
 
 #endif
 
+#ifndef		STDIN_FILENO
+#	define	STDIN_FILENO	0
+#endif
+
+#ifndef		IEXTEN
+#	define	IEXTEN		0
+#endif
+
+
 /*
 	#ifdef	NOERRNO
 	extern int errno;
@@ -120,14 +129,6 @@
 #define	TERMCAPSIZE	2048
 #define	TTYNAME		"/dev/tty"
 
-#ifndef		STDIN_FILENO
-#	define	STDIN_FILENO	0
-#endif
-
-#ifndef		IEXTEN
-#	define	IEXTEN		0
-#endif
-
 static	void	term_locate_flush();
 static	void	term_color_flush();
 static	void	term_tputs(const char *str);
@@ -142,6 +143,8 @@ static	u_char	ungetbuf[16];
 static	int 	ungetnum = 0;
 #endif
 
+extern	char *tparm(const char *str, ...);
+
 
 typedef	struct
 {
@@ -264,7 +267,8 @@ char	*term_getent(const char *s_id, const char *s_def)
 	return mem_strdup(cp);
 }
 
-void	term_tparam(char *s, const char *fmt, int arg1, int arg2)
+#if 0
+static	void	term_tparam(char *s, const char *fmt, int arg1, int arg2)
 {
 	while (*fmt!='\0')
 		{
@@ -279,7 +283,7 @@ void	term_tparam(char *s, const char *fmt, int arg1, int arg2)
 		 	{
 		  case 'd':
 		  	 sprintf(s, "%d", arg1);
-		  	 s+=strlen(s);
+		   	 s+=strlen(s);
 		  	 arg1=arg2;
 		  	 arg2=0;
 		  	 break;
@@ -287,14 +291,18 @@ void	term_tparam(char *s, const char *fmt, int arg1, int arg2)
 		  	 ++arg1;
 		  	 ++arg2;
 		  	 break;
+		  case 'p':
+		  	 ++fmt;
+		  	 break;
 		  case '%':
 		  	 *s++='%';
 		  	}
-		 ++fmt;
+		 if (*fmt!='\0')
+		 	++fmt;
 		}
 	*s='\0';
 }
-
+#endif
 
 void	term_queue_clear()
 {
@@ -452,6 +460,7 @@ void	term_init()
 	if (tgetent(tbuf, term.name) <= 0)
 		{
 //		 warning("No termcap presented.");
+		 report_puts("No termcap presented.");
 		 term.sizey=24;
 		 term.sizex=79;
 
@@ -508,6 +517,34 @@ void	term_init()
 		}
 }
 
+void	term_report()
+{
+	report_puts("ü:\n");
+	report_printf("  x = %d, y = %d\n", term.sizex, term.sizey);
+
+	report_puts("ɽ:\n");
+
+	report_printf("  term.t_init = %s\n", term.t_init);
+	report_printf("  term.t_end = %s\n", term.t_end);
+	report_printf("  term.t_keypad = %s\n", term.t_keypad);
+	report_printf("  term.t_nokeypad = %s\n", term.t_nokeypad);
+	report_printf("  term.t_normalcursor = %s\n", term.t_normalcursor);
+	report_printf("  term.t_hidecursor = %s\n", term.t_hidecursor);
+	report_printf("  term.t_vbell = %s\n", term.t_vbell);
+	report_printf("  term.t_clear = %s\n", term.t_clear);
+	report_printf("  term.t_normal = %s\n", term.t_normal);
+	report_printf("  term.t_bold = %s\n", term.t_bold);
+	report_printf("  term.t_reverse = %s\n", term.t_reverse);
+	report_printf("  term.t_underline = %s\n", term.t_underline);
+	report_printf("  term.end_underline = %s\n", term.end_underline);
+	report_printf("  term.l_clear = %s\n", term.l_clear);
+	report_printf("  term.l_insert = %s\n", term.l_insert);
+	report_printf("  term.l_delete = %s\n", term.l_delete);
+	report_printf("  term.ln_insert = %s\n", term.ln_insert);
+	report_printf("  term.ln_delete = %s\n", term.ln_delete);
+	report_printf("  term.cn_locate = %s\n", term.cn_locate);
+}
+
 void	term_kflush()
 {
 	tcflush(fileno(term.fp_tty), TCIFLUSH);
@@ -639,7 +676,6 @@ int 	term_regetch(c)
 }
 
 
-
 static	void	term_locate_flush()
 {
 	char	*p, *q, buf[LN_dispbuf+1];
@@ -743,6 +779,12 @@ void	term_puts(const char *s)
 {
 	int 	n;
 
+	if (term.y>=term.sizey)
+		return ;
+
+	if (term.x>0 && term.scr[term.y][term.x]==SCR_ignore)
+		fprintf(stderr, "+");
+
 	if (term.x>0 && term.scr[term.y][term.x]==SCR_ignore)
 		term.scr[term.y][term.x-1]= SCR_code(AC_normal, ' ');
 
@@ -896,8 +938,9 @@ static	void	term_scroll_dn(int n)
 
 	if (*term.ln_insert!='\0')
 		{
-		 term_tparam(buf, term.ln_insert, n, 0);
-		 term_tputs(buf);
+//		 term_tparam(buf, term.ln_insert, n, 0);
+//		 term_tputs(buf);
+		 term_tputs(tparm(term.ln_insert, n));
 		} else
 		{
 		 while (n-->0)
@@ -911,8 +954,9 @@ static	void	term_scroll_up(int n)
 
 	if (*term.ln_delete!='\0')
 		{
-		 term_tparam(buf, term.ln_delete, n, 0);
-		 term_tputs(buf);
+//		 term_tparam(buf, term.ln_delete, n, 0);
+//		 term_tputs(buf);
+		 term_tputs(tparm(term.ln_delete, n));
 		} else
 		{
 		 while (n-->0)
