@@ -71,6 +71,15 @@ typedef	struct
 static	esc_t	esc[MAX_esc];
 static	int 	esc_num=0;
 
+void	term_keyreport()
+{
+	int 	i;
+
+	report_puts(":\n");
+	for (i=0; i<esc_num; ++i)
+		report_printf("  [%s] -> %x\n",esc[i].seq, esc[i].scode);
+}
+
 
 void	term_escset(int n, const char *e, const char *d)
 {
@@ -95,6 +104,12 @@ void	term_escset(int n, const char *e, const char *d)
 }
 
 
+#if 0
+
+
+
+
+
 #define	MAX_keyring	64
 #define	MAX_skip	4
 
@@ -218,6 +233,158 @@ int 	term_inkey()
 
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+#else
+
+#define	MAX_keyring	64
+#define	MAX_skip	4
+
+int 	key_ring[MAX_keyring];
+int 	key_ring_in=0;
+int 	key_ring_out=0;
+
+int 	key_skip=0;
+
+
+void	key_ring_pushall(long n)
+{
+	int 	ch;
+
+	for (;;)
+		{
+		 if ((key_ring_in+1)%MAX_keyring == key_ring_out || !term_kbhit(n))
+		 	return;
+
+		 ch=term_getch();
+		 if (ch!=-1)
+		 	{
+		 	 key_ring[key_ring_in++]=ch;
+		 	 if (key_ring_in>=MAX_keyring)
+		 	 	key_ring_in=0;
+		 	}
+		}
+}
+
+int 	key_ring_pop()
+{
+	int 	ch;
+
+	while(key_ring_in == key_ring_out)
+		key_ring_pushall(1000L);
+
+	ch=key_ring[key_ring_out++];
+	if (key_ring_out>=MAX_keyring)
+		key_ring_out=0;
+
+	return ch;
+}
+
+void	key_ring_push(int ch)
+{
+	--key_ring_out;
+	if (key_ring_out<0)
+		key_ring_out=MAX_keyring;
+
+	key_ring[key_ring_out]=ch;
+}
+
+int 	term_inkey()
+{
+	int i, j, ch, key, start, end, s, e;
+
+
+	if (key_ring_in!=key_ring_out && key_skip<MAX_skip)
+		++key_skip; else
+		{
+		 key_skip=0;
+		 term_csr_flush();
+		}
+
+//term_csr_flush();
+
+	key = ch = key_ring_pop();
+
+	start = 0;
+	end   = esc_num;
+
+	for (i=0;;i++) 
+	{
+		s = start;
+		e = end;
+		start = end = -1;
+
+		for (j=s; j<e; j++)
+			{
+			 if (i>=esc[j].len || esc[j].seq[i]!=ch)
+			 	{
+			 	 if (end!=-1)
+			 	 	break;
+			 	} else
+			 	{
+			 	 if (esc[j].len == i+1)
+			 	 	return(esc[j].scode);
+
+		 	 	 if (start==-1)
+		 	 	 	start=j;
+			 	 end = j+1;
+			 	}
+			}
+		if (start==-1)
+			break;
+
+		if (key_ring_in==key_ring_out && !term_kbhit(WAITKEYPAD*1000L))
+			break;
+
+/*		if (key_ring_in==key_ring_out)
+			{
+			 key_ring_pushall(WAITKEYPAD * 1000L);
+			 if (key_ring_in==key_ring_out)
+			 	break;
+			}
+*/
+		ch = key_ring_pop();
+	}
+
+	if (i>0)
+		{
+		 key_ring_push(ch);
+		 while(--i>0)
+		 	key_ring_push(esc[j-1].seq[i]);
+		}
+	return (key);
+}
+
+#endif
+
+
+
+
 #define	KS_shift	0x10000
 #define	KS_ctrl		0x20000
 #define	KS_alt		0x40000
