@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
-  filer 0.2.6
+  filer 0.2.7
 
     Copyright (c) 1997,1998,1999,2000 SASAKI Shunsuke.
     All rights reserved. 
@@ -18,6 +18,80 @@
 #	endif
 #endif
 
+#if 0
+typedef	struct	fitem
+{
+	char			*fn;
+	char			*e;
+	struct	stat	stat;
+
+	struct	fitem	*next;
+}	fitem_t;
+
+typedef	struct
+{
+	fitem_t	*fitem;
+	int 	n;
+}	flist_t;
+
+typedef	struct
+{
+	char	path[LN_path +1];
+
+	menu_t	menu;
+	bool	df;
+
+	flist_t	flist;
+	fitem_t	**findex;
+	int 	*mark;
+	int 	markmax;
+	int 	marknum;
+
+	char	match[LN_path+1];
+}	fw_t;
+fw_t	fw[2];
+#define	fw_c	fw[eff.wa]
+
+typedef	struct
+{
+	int 	wn;			/* windowο */
+	int 	wa;			/* Window Active  */
+	bool	df;			/* dirty flag */
+
+	int 	sort[2];	/* sort */
+}	eff_t;
+eff_t	eff;
+
+enum	{SA_none,SA_fname,SA_ext};
+
+typedef struct	fop
+{
+	int 	(*file_func)(const char*,struct stat *,const char*, struct fop*);
+	int 	(*dir_func)(const char*,struct stat *,const char*
+														,bool, struct fop*);
+	char    *title;
+
+	int 	om;		/* overwrite mode. */
+	int 	pfm;	/* permition force mode. */
+
+	int 	en;		/* error number */
+}	fop_t;
+
+#endif
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 /* ǥ쥯ȥư */
@@ -46,15 +120,14 @@ void	dm_init()
 		dm[i].ct=0;
 }
 
-void	dm_set(const char *p,const char *name)
+void	dm_set(const char *p,const char *s)
 {
 	int 	i,a,b;
 	char	buf[LN_path+1];
 	char	path[LN_path+1];
 
-	strcpy(path,p);
-	reg_path("",path,TRUE);
-	strcpy(buf,name);
+	realpath(p, path);
+	strcpy(buf, s);
 	if (*buf!='\0'&& buf[strlen(buf)-1]=='/')
 		buf[strlen(buf)-1]='\0';
 
@@ -98,8 +171,7 @@ void	dm_get(const char *p,char *s)
 	int 	i;
 	char	path[LN_path+1];
 
-	strcpy(path,p);
-	reg_path("", path,TRUE);
+	realpath(p, path);
 
 	*s='\0';
 	for(i=0;i<dm_num;++i)
@@ -155,28 +227,15 @@ void	eff_init(const char *s1, const char *s2)
 
 
 
-void	fw_getstat(fw_t *fwp,int a,struct stat *stp)
+struct stat	*fw_getstat(fw_t *fwp,int a)
 {
 	if (a==-1)
 		a=fwp->menu.cy+fwp->menu.sy;
 //	min(a,fwp->flist.n);
 
-	memcpy(stp,&fwp->findex[a]->stat,sizeof (struct stat));
+	return &fwp->findex[a]->stat;
 }
 
-void	fw_getfn(fw_t *fwp,int a,char *s)
-{
-	char	*p;
-
-	if (a==-1)
-		a=fwp->menu.cy+fwp->menu.sy;
-//	min(a,fwp->flist.n);
-
-	strcpy(s,fwp->findex[a]->name);
-	p=fwp->findex[a]->ext;
-	if (*p!='\0')
-		sprintf(s+strlen(s),".%s",p);
-}
 
 fitem_t	*fw_getfi(fw_t *fwp,int a)
 {
@@ -188,7 +247,6 @@ fitem_t	*fw_getfi(fw_t *fwp,int a)
 void	fwc_chdir(const char *s,bool f)
 {
 	char	path[LN_path+1];
-	struct	stat	st;
 	bool	uf;
 
 	if (*s=='\0')
@@ -198,7 +256,7 @@ void	fwc_chdir(const char *s,bool f)
 		{
 		 uf=TRUE;
 		 sprintf(path,"%s..",fw_c.path);
-		 reg_path("", path,FALSE);
+		 reg_path(NULL, path,FALSE);
 		}else
 		{
 		 uf=FALSE;
@@ -208,12 +266,9 @@ void	fwc_chdir(const char *s,bool f)
 		}
 
 	reg_path(fw_c.path, path,FALSE);
-	if (stat(path,&st)==0 &&(st.st_mode&S_IFMT)==S_IFDIR)
+	if (dir_isdir(path))
 		{
-		 char	buf[LN_path+1];
-
-		 fw_getfn(&fw_c,-1,buf);
-		 dm_set(fw_c.path,buf);
+		 dm_set(fw_c.path, fw_c.findex[fw_c.menu.cy+fw_c.menu.sy]->fn);
 		 if (uf&& strcmp(fw_c.path,"/")!=0)
 		 	dm_set(path,fw_c.path+strlen(path));
 
@@ -233,19 +288,14 @@ void	fwc_chdir(const char *s,bool f)
 
 void	fw_match(fw_t *fwp)
 {
-	char	buf[LN_path+1];
 	int 	i;
 
-	for (i=0;i<fwp->flist.n;++i)
-		{
-		 fw_getfn(fwp,i,buf);
-		 if (strcmp(fwp->match,buf)==0)
-		 	{
-		  	 menu_csrmove(&fwp->menu,i);
-		 	 return;
-		 	}
-		}
-	menu_csrmove(&fwp->menu,0);
+	i=fwp->flist.n-1;
+
+	while (i>0 && strcmp(fwp->match, fwp->findex[i]->fn)!=0)
+		--i;
+
+	menu_csrmove(&fwp->menu, i);
 }
 
 
@@ -261,8 +311,7 @@ fitem_t	*fitem_free(fitem_t *fip)
 		return NULL;
 	next=fip->next;
 
-	free(fip->name);
-	free(fip->ext);
+	free(fip->fn);
 	free(fip);
 
 	return next;
@@ -282,43 +331,53 @@ void	fw_free(fw_t *fwp)
 	fwp->flist.n=0;
 }
 
-fitem_t	*fitem_mk(const char *path,const char *s)
+fitem_t	*fitem_mk(const char *path, char *s)
 {
-	char	buf[LN_path+1];
-	const char	*p,*q;
+	char	buf[LN_path+1], *p;
 	fitem_t	*fitem;
 
 	fitem=(fitem_t *)mem_alloc(sizeof(fitem_t));
-
-	q=get_ext(s);
-	if (q==NULL)
-		{
-		 p=s;
-		 q=""; 
-		} else
-		{
-		 strncpy(buf,s,q-s);
-		 buf[q-s]='\0';
-		 ++q;
-		 p=buf;
-		}
-
-	fitem->name=(char *)mem_alloc(strlen(p)+1);
-	strcpy(fitem->name,p);
-
-	fitem->ext=(char *)mem_alloc(strlen(q)+1);
-	strcpy(fitem->ext,q);
-
-	sprintf(buf,"%s/%s",path,s);
-	if (strcmp(s,"..")==0)
-		reg_path("", buf,FALSE);
-
-	stat(buf,&fitem->stat);
 	fitem->next=NULL;
 
+	fitem->fn=s;
+
+	p=dir_pext(s);
+	if (p==NULL)
+		fitem->e=s+strlen(s); else
+		fitem->e=p;
+
+	sprintf(buf,"%s/%s",path, s);
+	if (strcmp(s,"..")==0)
+		reg_path(NULL, buf, FALSE);
+	stat(buf, &fitem->stat);
+
 	return fitem;
 }
 
+
+#if 1
+
+void	flist_set(flist_t *flp,const char *path)
+{
+	char	**p;
+	fitem_t	*fitem;
+
+	p=dir_glob(path, TRUE);
+
+	fitem=  flp->fitem= fitem_mk(path, p[0]);
+	flp->n= 1;
+
+	while(p[flp->n]!=NULL)
+		{
+		 fitem->next= fitem_mk(path, p[flp->n]);
+		 fitem= fitem->next;
+		 ++flp->n;
+		}
+	free(p);
+}
+
+#else
+
 void	flist_set(flist_t *flp,const char *path)
 {
 	DIR 			*dirp;
@@ -345,8 +404,6 @@ void	flist_set(flist_t *flp,const char *path)
 		 	continue;
 
 		 next=fitem_mk(path, dp->d_name);
-		 if (next==NULL)
-		 	break;
 		 if (flp->fitem==NULL)
 		 	flp->fitem=next; else
 		 	fitem->next=next;
@@ -356,7 +413,9 @@ void	flist_set(flist_t *flp,const char *path)
 	closedir(dirp);
 }
 
-int 	findex_comp(const void *x,const void *y)
+#endif
+
+static	int 	findex_comp(const void *x,const void *y)
 {
 	fitem_t	*fi_x,*fi_y;
 	int 	i,j;
@@ -364,9 +423,9 @@ int 	findex_comp(const void *x,const void *y)
 	fi_x=*(fitem_t **)x;
 	fi_y=*(fitem_t **)y;
 
-	if (strncmp(fi_x->name,"..",2)==0)
+	if (strncmp(fi_x->fn,"..",2)==0)
 		return -1;
-	if (strncmp(fi_y->name,"..",2)==0)
+	if (strncmp(fi_y->fn,"..",2)==0)
 		return 1;
 
 	if ((fi_x->stat.st_mode&S_IFMT)== S_IFDIR&& (fi_y->stat.st_mode&S_IFMT)!=S_IFDIR)
@@ -374,9 +433,9 @@ int 	findex_comp(const void *x,const void *y)
 	if ((fi_x->stat.st_mode&S_IFMT)!= S_IFDIR&& (fi_y->stat.st_mode&S_IFMT)==S_IFDIR)
 		return 1;
 
-	if (*fi_x->name=='.'&& *fi_y->name!='.')
+	if (*fi_x->fn=='.'&& *fi_y->fn!='.')
 		return -1;
-	if (*fi_x->name!='.'&& *fi_y->name=='.')
+	if (*fi_x->fn!='.'&& *fi_y->fn=='.')
 		return 1;
 
 	for (i=0;i<2;++i)
@@ -387,10 +446,10 @@ int 	findex_comp(const void *x,const void *y)
 		  case SA_none:
 		  	 return 0;
 		  case SA_fname:
-		  	 j=strcmp(fi_x->name,fi_y->name);
+		  	 j=strcmp(fi_x->fn, fi_y->fn); //!! ̩Ƚ
 		  	 break;
 		  case SA_ext:
-		  	 j=strcmp(fi_x->ext,fi_y->ext);
+		  	 j=strcmp(fi_x->e ,fi_y->e);
 		  	 break;
 		 	}
 		 if (j!=0)
@@ -415,7 +474,7 @@ fitem_t	**findex_get(flist_t *flp)
 		 fitem=fitem->next;
 		}
 
-	qsort(findex,i,sizeof(fitem_t **),findex_comp);
+	qsort(findex, i, sizeof(fitem_t **), findex_comp);
 
 	return findex;
 }
@@ -439,22 +498,27 @@ void	prt_kmsize(char *s,off_t n)
 	sprintf(s,"%4d%c", (int)n, *scale);
 }
 
-void	fw_item_proc(int a,mitem_t *mip,void *vp)
+static	void	fw_item_proc(int a,mitem_t *mip,void *vp)
 {
 	char	buf[MAXLINESTR+1];
 	char	*s,*p;
-	struct tm	*tm;
 	fw_t	*fwp;
 
 	fwp=vp;
 	s=mip->str;
 
-	strjfcpy(s,fwp->findex[a]->name,MAXLINESTR,fwp->menu.drp->sizex-30);
-
-	p=fwp->findex[a]->ext;
+	strjfcpy(s, fwp->findex[a]->fn, MAXLINESTR, fwp->menu.drp->sizex-30);
+	p=fwp->findex[a]->e;
 	if (*p=='\0')
 		strcat(s,"     "); else
 		{
+		 int 	m, n;
+
+		 n=strlen(s);
+		 m=fwp->findex[a]->e - fwp->findex[a]->fn-1;
+		 if (m<n)
+		 	memset(s+m, ' ', n-m);
+
 		 strcat(s,".");
 		 strjfcpy(s+strlen(s),p,MAXLINESTR,4); // !MAXLINESTR
 		}
@@ -467,17 +531,15 @@ void	fw_item_proc(int a,mitem_t *mip,void *vp)
 		 mip->nc=sysinfo.c_eff_dirn;
 		}else
 		{
-		 prt_kmsize(buf,fwp->findex[a]->stat.st_size);
-		 strcat(s,buf);
+		 prt_kmsize(buf, fwp->findex[a]->stat.st_size);
+		 strcat(s, buf);
 
 		 mip->cc=sysinfo.c_eff_normc;
 		 mip->nc=sysinfo.c_eff_normn;
 		}
 
-	tm=localtime(&fwp->findex[a]->stat.st_mtime);
-	strftime(buf,15,"%y/%m/%d %R",tm);
-	strcat(s," ");
-	strcat(s,buf);
+	strftime(buf,15,"%y/%m/%d %R", localtime(&fwp->findex[a]->stat.st_mtime));
+	sprintf(s+strlen(s), " %s",buf);
 }
 
 void	fw_make(fw_t *fwp)
@@ -548,7 +610,7 @@ void	fw_chmark(fw_t *fwp,int a)
 
 	n= a!=-1? a: fwp->menu.cy+fwp->menu.sy;
 
-	if (strcmp(fwp->findex[n]->name,"..")==0 && *fwp->findex[n]->ext=='\0')
+	if (strcmp(fwp->findex[n]->fn,"..")==0)
 		return;
 
 	if (fwp->mark[n]==0)
@@ -566,15 +628,13 @@ void	fw_chmark(fw_t *fwp,int a)
 
 void	fw_chmarkall(fw_t *fwp)
 {
-	struct	stat	st;
 	int	i;
 
 	if (fwp->marknum==0)
 		{
 		 for (i=0;i<fwp->flist.n;++i)
 		 	{
-		 	 fw_getstat(fwp, i, &st);
-		 	 if ((st.st_mode &S_IFMT)!=S_IFDIR)
+		 	 if ((fw_getstat(fwp, i)->st_mode &S_IFMT)!=S_IFDIR)
 		 	 	fw_chmark(fwp, i);
 		 	}
 		} else
@@ -586,7 +646,7 @@ void	fw_chmarkall(fw_t *fwp)
 
 /* file opration */
 
-bool	select_readonly(fop_t *fop)
+static	bool	select_readonly(fop_t *fop)
 {
 	int 	res;
 
@@ -599,6 +659,7 @@ bool	select_readonly(fop_t *fop)
 					, 4, "Y) ", "N) ʤ"
 					,"A) ʹƽ", "K) ʹƽʤ");
 
+
 	RefreshMessage();
 //	eff.df=TRUE;
 
@@ -642,9 +703,9 @@ int 	fw_fop_file(const char *srcpath,const char *fn,struct stat *srcstp
 
 		 	 	 if (fop->om==0)
 		 	 	 	{
-					 fop->om =1+ menu_vselect(term_sizex()/2 ,term_sizey()/2
-					 , 4, "O) ƾ", "U) դ"
-					 , "N) Ʊ̾ʣ̤ʤ", "R) ̾Ѥ");
+					 fop->om=1+menu_vselect(term_sizex()/2, term_sizey()/2, 4
+					 	, "O) ƾ", "U) դ"
+					 	, "N) Ʊ̾ʣ̤ʤ", "R) ̾Ѥ");
 
 //		 	 	 	 eff.df=TRUE;
 		 	 	 	}
@@ -680,8 +741,8 @@ int 	fw_fop_file(const char *srcpath,const char *fn,struct stat *srcstp
 		 if (res!=FR_err)
 		 	return res;
 
-		 res=menu_vselect(term_sizex()/2
-		 	, term_sizey()/2, 3, "A) ", "R) ƻ", "S) ³");
+		 res=menu_vselect(term_sizex()/2, term_sizey()/2, 3,
+		 	"A) ", "R) ƻ", "S) ³");
 
 //		 eff.df=TRUE;
 
@@ -732,8 +793,8 @@ int 	fw_fop_dir(const char *srcpath,const char *fn,struct stat *srcstp
 		 res=fop->dir_func(srcfn, srcstp, dstfn, wf, fop);
 		 if (res!=FR_err)
 		 	return res;
-		 res=menu_vselect(term_sizex()/2
-		 	,term_sizey()/2, 3, "A) ", "R) ƻ", "S) ³");
+		 res=menu_vselect(term_sizex()/2, term_sizey()/2, 3,
+		 	"A) ", "R) ƻ", "S) ³");
 
 //		 eff.df=TRUE;
 
@@ -756,7 +817,7 @@ int 	fw_fop_list(fitem_t **findex, size_t fi_nums
 {
 	struct	stat	*srcstp;
 	int 	res;
-	char	*p,srcfn[LN_path+1];
+	char	*p;
 	int 	i;
 
 	fitem_t	**fip;
@@ -769,26 +830,21 @@ int 	fw_fop_list(fitem_t **findex, size_t fi_nums
 	for (i=0;i<fi_nums;++i)
 		{
 		 srcstp=&findex[i]->stat;
-		 strcpy(srcfn, findex[i]->name);
-		 p=findex[i]->ext;
-		 if (*p!='\0')
-		 	sprintf(srcfn+strlen(srcfn),".%s",p);
-
-//fprintf(stderr,"%03d:%s%s -> %s \n",i, srcpath,srcfn,dstpath);
+//fprintf(stderr,"%03d:%s%s -> %s \n",i, srcpath,findex[i]->fn,dstpath);
 
 		 if ((srcstp->st_mode& S_IFMT)!=S_IFDIR)
-			res=fw_fop_file(srcpath, srcfn, srcstp, dstpath, fop); else
+			res=fw_fop_file(srcpath, findex[i]->fn, srcstp, dstpath, fop); else
 			{
-			 if (strcmp(srcfn,".")==0|| strcmp(srcfn,"..")==0)
+			 if (strcmp(findex[i]->fn,".")==0|| strcmp(findex[i]->fn,"..")==0)
 			 	continue;
 
 
-			 sprintf(srctmp,"%s%s/",srcpath,srcfn);
+			 sprintf(srctmp,"%s%s/",srcpath,findex[i]->fn);
 			 if (dstpath!=NULL)
-			 	sprintf(dsttmp,"%s%s/",dstpath,srcfn);
+			 	sprintf(dsttmp,"%s%s/",dstpath,findex[i]->fn);
 			 system_msg(srctmp);
 
-			 res=fw_fop_dir(srcpath, srcfn, srcstp, dstpath, fop, TRUE);
+			 res=fw_fop_dir(srcpath, findex[i]->fn, srcstp, dstpath, fop, TRUE);
 			 if (res==FR_err)
 			 	return FR_err;
 			 if (res==FR_nonrec)
@@ -808,7 +864,7 @@ int 	fw_fop_list(fitem_t **findex, size_t fi_nums
 		 	 	fitem=fitem_free(fitem);
 
 
-			 res=fw_fop_dir(srcpath, srcfn, srcstp, dstpath, fop, FALSE);
+			 res=fw_fop_dir(srcpath, findex[i]->fn, srcstp, dstpath, fop, FALSE);
 		 	}
 		 if (res==FR_err|| res==FR_end)
 		 	return res;
@@ -865,7 +921,7 @@ void	fw_fop(fw_t *srcfwp, const char *dstpath
 }
 
 
-int		cp_proc_dir(const char *src,struct stat *srcstp,const char *dst
+static	int		cp_proc_dir(const char *src,struct stat *srcstp,const char *dst
 			,bool wf,fop_t *fop)
 {
 	if (!wf)
@@ -882,7 +938,7 @@ int		cp_proc_dir(const char *src,struct stat *srcstp,const char *dst
 
 #define	MAX_cpbuf	4096
 
-int		cp_proc_file(const char *src,struct stat *srcstp
+static	int		cp_proc_file(const char *src,struct stat *srcstp
 								,const char *dst,fop_t *fop)
 {
 	FILE	*fpr,*fpw;
@@ -935,7 +991,7 @@ int		cp_proc_file(const char *src,struct stat *srcstp
 	return FR_ok;
 }
 
-int		mv_proc_dir(const char *src,struct stat *srcstp,const char *dst
+static	int		mv_proc_dir(const char *src,struct stat *srcstp,const char *dst
 			,bool wf,fop_t* fop)
 {
 	if (!wf)
@@ -953,7 +1009,7 @@ int		mv_proc_dir(const char *src,struct stat *srcstp,const char *dst
 	return cp_proc_dir(src,srcstp,dst,wf,fop);
 }
 
-int 	mv_proc_file(const char *src,struct stat *srcstp
+static	int 	mv_proc_file(const char *src,struct stat *srcstp
 								,const char *dst,fop_t *fop)
 {
 	int 	res;
@@ -966,7 +1022,7 @@ int 	mv_proc_file(const char *src,struct stat *srcstp
 	return res;
 }
 
-int 	rm_proc_file(const char *src,struct stat *srcstp
+static	int 	rm_proc_file(const char *src,struct stat *srcstp
 									,const char *dst,fop_t *fop)
 {
 	if (access(src,W_OK)<0&& !select_readonly(fop))
@@ -977,7 +1033,7 @@ int 	rm_proc_file(const char *src,struct stat *srcstp
 	return FR_err;
 }
 
-int 	rm_proc_dir(const char *src,struct stat *srcstp,const char *dst
+static	int 	rm_proc_dir(const char *src,struct stat *srcstp,const char *dst
 			,bool wf,fop_t* fop)
 {
 	if (wf)
@@ -993,12 +1049,12 @@ bool	fw_cpdest(char *s, fw_t *srcfwp, fw_t *dstfwp)
 	char	srcpath[LN_path+1];
 
 	strcpy(srcpath,srcfwp->path);
-	reg_path("", srcpath,TRUE);	// ??
+	reg_path(NULL, srcpath,TRUE);	// ??
 
 	if (dstfwp!=NULL)
 		{
 		 strcpy(s,dstfwp->path);
-		 reg_path("", s,TRUE);
+		 reg_path(NULL, s,TRUE);
 //fprintf(stderr,"dst [%s]\n",s);
 		 if (strcmp(srcpath,s)!=0)	// !!ʬβɤåԤ
 		 	return TRUE;
@@ -1054,16 +1110,15 @@ void	fw_remove(fw_t *srcfwp,fw_t *dstfwp)
 
 void	fw_rename(fw_t *fwp)
 {
-	char	buf[LN_path+1],fn[LN_path+1],tmp[LN_path+1];
+	char	buf[LN_path+1], tmp[LN_path+1];
+	char	*fn;
 
-	getcwd(buf,LN_path);
-
-	fw_getfn(fwp, -1, fn);
-
-	strcpy(tmp,fn);
+	fn=fw_c.findex[fw_c.menu.cy+fw_c.menu.sy]->fn;
+	strcpy(tmp, fn);
 	if (GetS("Rename :",tmp)==ESCAPE)
 		return;
 
+	getcwd(buf, LN_path);
 	chdir(fwp->path);
 	if (rename(fn, tmp)==0)
 		dm_set(fwp->path,tmp);
@@ -1118,7 +1173,7 @@ char	*pval_parse(const char *s, char *t)
 //	 case 'm':
 //		 if (fw_c.marknum>0)
 	 case 'f':
-	 	 fw_getfn(&fw_c, -1, t);
+		 strcpy(t, fw_c.findex[fw_c.menu.cy+fw_c.menu.sy]->fn);
 	 	 break;
 		}
 
@@ -1189,7 +1244,7 @@ bool	eff_exec()
 bool	eff_filer(char *fn)
 {
 	int 	c;
-	char	buf[LN_path+1];
+	char	buf[LN_path+1], *p;
 	bool	f;
 
 //	term_cls();
@@ -1201,6 +1256,7 @@ bool	eff_filer(char *fn)
 
 	if (eff.wn==1)
 		dsp_regrm(fw[1].menu.drp);
+
 	for (;;)
 		{
 		 dsp_allview();
@@ -1250,8 +1306,8 @@ bool	eff_filer(char *fn)
 		  	 	continue;
 
 		  case KF_EffReRead:
-		  	 fw_getfn(&fw_c,-1,buf);
-		  	 dm_set(fw_c.path,buf);
+		  	 dm_set(fw_c.path, fw_c.findex[fw_c.menu.cy+fw_c.menu.sy]->fn);
+
 		  	 fw_make(&fw[eff.wa]);
 		  	 term_cls();
 		  	 continue;
@@ -1318,23 +1374,22 @@ bool	eff_filer(char *fn)
 		  	 continue;
 
 		  case KF_EffReturn:
-		  	 fw_getfn(&fw_c,-1,buf);
+		 	 p=fw_c.findex[fw_c.menu.cy+fw_c.menu.sy]->fn;
 
 		  	 if ((fw_c.findex[fw_c.menu.cy+fw_c.menu.sy]->stat.st_mode &S_IFMT)
 		  	 		==S_IFDIR)
 		  	 	{
-		  	 	 fwc_chdir(buf,FALSE);
+		  	 	 fwc_chdir(p, FALSE);
 		  	 	 fw_make(&fw[eff.wa]);
 //				 term_cls();
 		  	 	 continue;
 		  	 	}
-		  	 dm_set(fw_c.path,buf);
-		  	 sprintf(fn,"%s%s",fw_c.path,buf);
+		  	 dm_set(fw_c.path, p);
+		  	 sprintf(fn,"%s%s",fw_c.path, p);
 		  	 f=TRUE;
 		  	 break;
 		  case KF_EffEscape:
-		  	 fw_getfn(&fw_c,-1,buf);
-		  	 dm_set(fw_c.path,buf);
+		  	 dm_set(fw_c.path, fw_c.findex[fw_c.menu.cy+fw_c.menu.sy]->fn);
 		  	 f=FALSE;
 		  	 break;
 		  	}
@@ -1362,8 +1417,6 @@ bool	eff_filer(char *fn)
 */
 bool	need_filer( const char* pszFilename )
 {
-	struct stat	st ;
-
 	if (*pszFilename=='\0')
 		return TRUE;
 
@@ -1376,6 +1429,6 @@ bool	need_filer( const char* pszFilename )
 		return TRUE ;
 
 	/*	if it is a directory, yes. */
-	return (stat(pszFilename, &st) == 0 && (st.st_mode & S_IFDIR) != 0) ;
+	return dir_isdir(pszFilename);
 }
 
