@@ -13,8 +13,8 @@
 
 typedef	struct	fitem
 {
-	char			*name;
-	char			*ext;
+	char			*fn;
+	char			*e;
 	struct	stat	stat;
 
 	struct	fitem	*next;
