@@ -384,7 +384,7 @@ void	SeeOption()
 
 	do {
 		menu_iteminit(&menu);
-		menu_itemmake(&menu,option_set_proc,13,NULL);
+		menu_itemmake(&menu, option_set_proc, 13, NULL);
 		res=menu_select(&menu);
 		menu_itemfin(&menu);
 
@@ -515,14 +515,13 @@ void	config_read(char *path)
 		 fp=fopen(buf, "r");
 		 if (fp==NULL)
 		 	{
-		 	 sprintf(buf, "%s/%s", SYSCONFDIR, path);
+		 	 sprintf(buf, SYSCONFDIR "/%s", path);
 		 	 fp=fopen(buf, "r");
 		 	 if (fp==NULL)
 		 	 	return;
 		 	}
 // fprintf(stderr, "*[%s]\n", buf);
 		}
-		
 
 	reg_chr='\0';
 	*zone_buf='\0';
@@ -572,7 +571,7 @@ void	config_read(char *path)
 
 		 	 if (mode==1)
 		 	 	{
-		 	 	 if (isalpha(c))
+		 	 	 if (c>='A' && c<='z' && isalpha(c))
 		 	 	 	{
 		 	 	 	 com_chr=tolower(c);
 		 	 	 	 mode=0;
@@ -798,7 +797,7 @@ void	config_read(char *path)
 	fclose(fp);
 }
 
-/*
+#if 0
 
 char	*dval_extract(const char *s, char *t) // Ȥꤢ
 {
@@ -876,4 +875,5 @@ char	*dval_extract(const char *s, char *t) // Ȥꤢ
 	return (char *)s;
 }
 
-*/
+#endif
+
